#include "PyEMClasses.h"
#include "Cylinder.h"
#include <boost/python.hpp>
#include <boost/python/to_python_converter.hpp>

namespace python = boost::python;

BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(Cylinders_WriteDejaVu, WriteDejaVu, 1, 4);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(Cylinders_WriteCosec, WriteCosec, 1, 7);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(Cylinders_OrientationDiff, OrientationDiff, 1, 3);

void export_Cylinders()
{
    
    python::class_<CylLenComp> CylLenComp_class("CylLenComp", "Cylinder length compare class");
    python::class_<CylWeightComp> CylWeightComp_class("CylWeightComp", "Cylinder weight compare class");
    
    python::class_<Cylinders> Cylinders_class("Cylinders", "A container of Cylinder objects");

    Cylinders_class.def("size", &Cylinders::size, 
			"int size() \n Returns number of Cylinders.");
    Cylinders_class.def("copy", &Cylinders::copy, 
			"Cylinders copy()");
    Cylinders_class.def("subset", &Cylinders::subset, 
			"Cylinders subset(list1, int n)");
    Cylinders_class.def("add", &Cylinders::add, 
			"bool add(Cylinder cyl)");
    
    Cylinders_class.def("ReadListfile", (bool (Cylinders::*)(const char*)) &Cylinders::ReadListfile,
			"bool ReadListfile(string in = \"-\");");

    Cylinders_class.def("ReadListfile", (bool (Cylinders::*)(FILE*)) &Cylinders::ReadListfile,
			"bool ReadListfile(FILE in = stdin);");

    Cylinders_class.def("WriteListfile", (bool (Cylinders::*)(const char*)) &Cylinders::WriteListfile,
			"bool WriteListfile(string out = \"-\");");

    Cylinders_class.def("WriteListfile", (bool (Cylinders::*)(FILE*)) &Cylinders::WriteListfile,
			"bool WriteListfile(FILE out = stdout);");

    Cylinders_class.def("ReadInventor", (bool (Cylinders::*)(const char*)) &Cylinders::ReadInventor,
			"bool ReadInventor(string in = \"-\");");

    Cylinders_class.def("ReadInventor", (bool (Cylinders::*)(FILE*)) &Cylinders::ReadInventor,
			"bool ReadInventor(FILE in = stdin);");

    Cylinders_class.def("WriteInventor", (bool (Cylinders::*)(const char*)) &Cylinders::WriteInventor,
			"bool WriteInventor(string out = \"-\");");

    Cylinders_class.def("WriteInventor", (bool (Cylinders::*)(FILE*)) &Cylinders::WriteInventor,
			"bool WriteInventor(FILE out = stdout);");

    Cylinders_class.def("WriteVRML", (bool (Cylinders::*)(const char*)) &Cylinders::WriteVRML,
			"bool WriteVRML(string out = \"-\");");

    Cylinders_class.def("WriteVRML", (bool (Cylinders::*)(FILE*)) &Cylinders::WriteVRML,
			"bool WriteVRML(FILE out = stdout);");

    Cylinders_class.def("ReadDejaVu", (bool (Cylinders::*)(const char*)) &Cylinders::ReadDejaVu,
			"bool ReadDejaVu(string in=\"-\");");

    Cylinders_class.def("ReadDejaVu", (bool (Cylinders::*)(FILE*)) &Cylinders::ReadDejaVu,
			"bool ReadDejaVu(FILE in=stdin);");

    Cylinders_class.def("WriteDejaVu", (bool (Cylinders::*)(FILE*, const char*,const char*, const char*)) &Cylinders::WriteDejaVu,
			Cylinders_WriteDejaVu("bool WriteDejaVu(outfile, mol = '0dum', note = 'dummy protein', pdb = '0dum.pdb')\nbool WriteDejaVu(outfilename, mol = '0dum', note = 'dummy protein', pdb = '0dum.pdb')"));

    Cylinders_class.def("WriteDejaVu", (bool (Cylinders::*)(const char*, const char*,const char*, const char*)) &Cylinders::WriteDejaVu,
			Cylinders_WriteDejaVu("bool WriteDejaVu(outfile, mol = '0dum', note = 'dummy protein', pdb = '0dum.pdb')\nbool WriteDejaVu(outfilename, mol = '0dum', note = 'dummy protein', pdb = '0dum.pdb')"));
    
    Cylinders_class.def("ReadCosec", (bool (Cylinders::*)(const char*)) &Cylinders::ReadCosec,
			"bool ReadCosec(string in =\"-\")");

    Cylinders_class.def("ReadCosec", (bool (Cylinders::*)(FILE*)) &Cylinders::ReadCosec,
			"bool ReadCosec(FILE in=stdin);");

    Cylinders_class.def("WriteCosec", (bool (Cylinders::*)(FILE*,const char*, const char*, const char*, const char*,const char*, const char*)) &Cylinders::WriteCosec,
			Cylinders_WriteCosec("bool WriteCosec(outfile, prog='helixhunter', pdbid='0dum', header='dummy protein', compnd='dummy protein', source='nowhere', author=''\n\nbool WriteCosec(outfilename, prog='helixhunter', pdbid='0dum', header='dummy protein', compnd='dummy protein', source='nowhere', author='')"));
    
    Cylinders_class.def("WriteCosec", (bool (Cylinders::*)(const char*,const char*, const char*,const char*, const char*,const char*, const char*)) &Cylinders::WriteCosec,
			Cylinders_WriteCosec("bool WriteCosec(outfile, prog='helixhunter', pdbid='0dum', header='dummy protein', compnd='dummy protein', source='nowhere', author=''\n\nbool WriteCosec(outfilename, prog='helixhunter', pdbid='0dum', header='dummy protein', compnd='dummy protein', source='nowhere', author='')"));
    
    
    Cylinders_class.def("SortLength", &Cylinders::SortLength, 
			"bool SortLength()");
    Cylinders_class.def("SortWeight", &Cylinders::SortWeight, 
			"bool SortWeight()");
    Cylinders_class.def("OrientationDiff", &Cylinders::OrientationDiff,
			Cylinders_OrientationDiff("float OrientationDiff(Cylinders comp, ignoredir=true, uselengthweight=true)\n  average individual cylinders angle diff"));

    Cylinders_class.def("mirror", &Cylinders::mirror, 
			"mirror()");
    Cylinders_class.def("Transform", (void (Cylinders::*)(Quater&, Vect&, Vect&))&Cylinders::Transform, 
			"Transform(Quater rot, Vect trans, Vect center)\n  transform: rotate then translate\n\nTransform(Euler rot, Vect trans, Vect center)\n  transform: rotate then translate");
    Cylinders_class.def("Transform", (void (Cylinders::*)(Euler&, Vect&, Vect&))&Cylinders::Transform, 
			"Transform(Quater rot, Vect trans, Vect center)\n  transform: rotate then translate\n\nTransform(Euler rot, Vect trans, Vect center)\n  transform: rotate then translate");
    Cylinders_class.def("Print", &Cylinders::print, "void print()");
    

}

    
