#include "PyEMClasses.h"

#include "PyUtil.h"
#include "PyEMData.h"
#include "Cylinder.h"
#include <boost/python.hpp>
#include <boost/python/to_python_converter.hpp>

namespace python = boost::python;


void export_ObjectStat()
{
    python::class_<ObjectStat> ObjectStat_class("ObjectStat", "ObjectStat");

    ObjectStat_class.def("SetObject", (void (ObjectStat::*)(EMData*, const std::vector<Vect*>&))&ObjectStat::SetObject, 
			 "SetObject(EMData map, list obj)\n  obj: a list of Vect ojbects\n\nSetObject(EMData map)");
    ObjectStat_class.def("SetObject", (void (ObjectStat::*)(EMData*))&ObjectStat::SetObject, 
			 "SetObject(EMData map, list obj)\n  obj: a list of Vect ojbects\n\nSetObject(EMData map)");
    ObjectStat_class.def("EigenSortTo", &ObjectStat::EigenSortTo, 
			 "EigenSortTo(ObjectStat obj)");
    ObjectStat_class.def("FitEllipsoid", &ObjectStat::FitEllipsoid, 
			 "FitEllipsoid(sort = true)");
    ObjectStat_class.def("Norm", &ObjectStat::Norm, 
			 "Norm()");
    ObjectStat_class.def("ComputeLongAxis", &ObjectStat::ComputeLongAxis, 
			 "ComputeLongAxis()");
    ObjectStat_class.def("Print", &ObjectStat::print, "print()");

}

void export_Helix()
{
    python::class_<Helix> Helix_class("Helix", "definitons for output format for use in IRIS Explorer module");

    Helix_class.def_readwrite("qx", &Helix::qx);
    Helix_class.def_readwrite("qy", &Helix::qy);
    Helix_class.def_readwrite("qz", &Helix::qz);
    Helix_class.def_readwrite("qa", &Helix::qa);
    Helix_class.def_readwrite("tx", &Helix::tx);
    Helix_class.def_readwrite("ty", &Helix::ty);
    Helix_class.def_readwrite("tz", &Helix::tz);
    Helix_class.def_readwrite("c", &Helix::c);
    Helix_class.def_readwrite("len", &Helix::len);
}


void export_util()
{
    python::def("tmpnam2", tmpnam2, "string tmpnam2()");
    python::def("resolv", resolv, "unsigned long resolv(string host = NULL)");
    python::def("sock_open", sock_open, "int sock_open(unsigned long addr, unsigned short port)");
    python::def("writesock", writesock, "int writesock(int sock, string data, int len)");
    python::def("FSsend", FSsend, "int FSsend(int command, int datalen, string filename, string data,int fileno=0)");
    python::def("maxf", (float (*)(float, float)) maxf, "float maxf(float f1, float f2)");
    python::def("maxf", (float (*)(float, float, float)) maxf, "float maxf(float f1,float f2,float f3)");
    python::def("maxf", (float (*)(float, float, float, float)) maxf, "float maxf(float f1,float f2,float f3,float f4)");
    python::def("minf", (float (*)(float, float, float)) minf, "float minf(float f1,float f2,float f3=1.0e30)");
    python::def("minf", (float (*)(float, float, float, float)) minf, "float minf(float f1,float f2,float f3,float f4)");
    python::def("floor2", floor2, "int floor2(float x)");
    python::def("hypot2", hypot2, "float hypot2(float x, float y)");
    python::def("hypot3", hypot3, "float hypot3(float x,float y,float z)");

    python::def("addHTMLSection", addHTMLSection, "void addHTMLSection(string sourcefile, FILE out, string secid)");
    python::def("error", (void (*)(char*, int))error, "void error(string msg, int pri)");
    python::def("error", (void (*)(int, char*))error, "void error(int pri, string msg)");
  
    python::def("file_lock_wait", file_lock_wait, "int file_lock_wait(FILE file)");
    python::def("file_lock", file_lock, "int file_lock(FILE file)");
    python::def("file_unlock", file_unlock, "void file_unlock(FILE file)");

    python::def("LOG", LOG, "void LOG(int ref, string filename, int mode, string text)");
    
    python::def("LOGbegin", py_LOGbegin, "int LOGbegin(list argv, int ppid=0)");
    
    python::def("LOGerror", LOGerror, "void LOGerror(int ref, int err, string text)");
    
    python::def("appinit", py_appinit, "void appinit(list argv) \n  All applications should call this before parsing the command line");
    python::def("appusage", appusage, "void appusage(string argv0)");
    python::def("frand", frand, "float frand(float min, float max)");
    python::def("grand", grand, "float grand(float mean, float sigma)");
    python::def("cyc", cyc, "int cyc(int i, int n)");
    python::def("interpolate", py_interpolate, "float interpolate(float xl, int n, list x_list, list y_list)");
    python::def("bilin", bilin, "float bilin(float p1, float p2, float p3, float p4, float t, float u)");
    
    python::def("p5peak", p5peak, "float p5peak(float v1, float v2, float v3, float v4,float v5) \n this will interpolate a peak location. v3 is considered to be at x=0. The other points are nearest neighbors returns 0 if not well defined");

    python::def("agauss", agauss, "float agauss(float a, float dx, float dy, float dz, float d)");
    python::def("SQR", SQR, "float SQR(float x)");
    python::def("roteulertb", py_roteulertb, "void py_roteulertb(Float alt, Float az, Float phi, float dx, float dy)");
    python::def("npfa", npfa, "int npfa(int low)");
    python::def("isvalidfft", isvalidfft, "int isvalidfft(int low)");
    python::def("bestfftsize", bestfftsize, "int bestfftsize(int low) \n returns a good fft size >= than low");
    
    python::def("save_data", (void (*)(PyList, PyList, int, const char*)) py_save_data, "void py_save_data(list x_list, list y_list, int n, string filename);");
    
    python::def("save_data", (void (*)(float, float, PyList, int, const char*)) py_save_data, "void py_save_data(float x0, float dx, list y_list, int n, string filename);");
    
    python::def("lsqfit", py_lsqfit, "lsqfit(int n, list dx, list dy, Float m, Float b, int ign0 = 1) \n least squares fit returns slope, m, and intercept, b");
    
    python::def("run", run, "int run(string program);");
}

void export_enums()
{
    python::enum_<ImageType>("Image")
	.value("mrc", IMAGE_MRC)
	.value("spider", IMAGE_SPIDER)
	.value("img", IMAGE_IMAGIC)
	.value("pgm", IMAGE_PGM)
	.value("lst", IMAGE_LST)
	.value("pif", IMAGE_PIF)
	.value("png", IMAGE_PNG)
	.value("hdf5", IMAGE_HDF)
	.value("dm3", IMAGE_DM3)
	.value("tif", IMAGE_TIF)
	.value("vtk", IMAGE_VTK)
	.value("sal", IMAGE_SAL)
	.value("map", IMAGE_ICOS)
	.value("img3d", IMAGE_IMAGIC3D)
	.value("emim", IMAGE_EMIM)
	.value("single_spider", IMAGE_SINGLE_SPIDER)
	.value("dm2", IMAGE_GATAN)
	.value("amira", IMAGE_AMIRA)
	.value("xplor", IMAGE_XPLOR)
	.value("any", IMAGE_ANY)
	;
}
