#include "ImageMxCon.h"
#include "ImHist.h"
#include <stdlib.h>

#define Inherited ImageMxConData

ImageMxCon::ImageMxCon(QWidget* parent,const char* name):Inherited( parent, name ) {
brt->setMin(-1.0);
brt->setMax(1.0);
brt->setValue(0.0);
cont->setMin(0);
cont->setMax(1.0);
cont->setValue(0.2);
connect(brt,SIGNAL(newValue(float)),SLOT(newBC(float)));
connect(cont,SIGNAL(newValue(float)),SLOT(newBC(float)));
connect(invert,SIGNAL(toggled(bool)),SLOT(newInv(bool)));
connect(label,SIGNAL(toggled(bool)),SLOT(newLab(bool)));
connect(label2,SIGNAL(toggled(bool)),SLOT(newLab2(bool)));
connect(bsaveFile,SIGNAL(clicked()),SLOT(doSaveFile()));
connect(bsave,SIGNAL(clicked()),SLOT(doSaveView()));
connect(bprint,SIGNAL(clicked()),SLOT(doPrint()));
connect(modeApp,SIGNAL(clicked()),SLOT(setModeApp()));
connect(modeDel,SIGNAL(clicked()),SLOT(setModeDel()));
connect(modeMove,SIGNAL(clicked()),SLOT(setModeMove()));
connect(modeSplit,SIGNAL(clicked()),SLOT(setModeSplit()));
connect(modeProbe,SIGNAL(clicked()),SLOT(setModeProbe()));
connect(scale,SIGNAL(returnPressed()),SLOT(newScale()));
connect(showhist,SIGNAL(toggled(bool)),SLOT(newHist(bool)));
connect(showfft,SIGNAL(toggled(bool)),SLOT(newFFT(bool)));
loc->setAlignment(AlignCenter);
data=NULL;
datal=NULL;
histvis=0;
subim=-1;
}


ImageMxCon::~ImageMxCon() {

}

void ImageMxCon::setup(float b,float c, int inv, int lab,int mode) {
	brt->setValue((int)(b));
	cont->setValue((int)(c));
	if (invert<0) invert->hide();
	else invert->setChecked(inv);
	if (lab<0) { label->hide(); label2->hide(); }
	else {
		label->setChecked(lab&1);
		label2->setChecked((lab&2)/2);
	}
	if (mode<0) {
		modeDel->setEnabled(FALSE);
		modeMove->setEnabled(FALSE);
		modeSplit->setEnabled(FALSE);
		bsaveFile->hide();
	}
	else {
		if (mode==0) modeApp->setOn(TRUE);
		else if (mode==1) modeDel->setOn(TRUE);
		else if (mode==2) modeMove->setOn(TRUE);
		else if (mode==3) modeSplit->setOn(TRUE);
		else if (mode==4) modeProbe->setOn(TRUE);
	}
}
	
void ImageMxCon::doPrint() {
emit print();
}

void ImageMxCon::doSaveFile() {
emit saveFile();
}

void ImageMxCon::doSaveView() {
emit saveImage();
}

void ImageMxCon::doProbe(unsigned char *mx,int Subim, int x,int y,float val) {
if (!histvis) showhist->toggle();

if (Subim!=subim && datal) {
	subim=Subim;
	imagesChanged(datal);
}

mini->setData(mx,7,7);
QString tmp;
tmp.sprintf("%1d,%1d",x,y);
loc->setText(tmp);
hist->setSel(val);
}

void ImageMxCon::newBC(float B) {
float b=(float)brt->Value();
float c=(float)cont->Value();
emit(setBC(b,c));
//printf("%f %f\n",b,c);
}

void ImageMxCon::newScale() {
float s=atof(scale->text());
if (s>0) emit setScale(s);
}

void ImageMxCon::newInv(bool t) {
emit(invToggle((int)t));
}

void ImageMxCon::newFFT(bool t) {
emit(fftToggle((int)t));
}

void ImageMxCon::newLab(bool l) {
emit(labToggle((int)l));
}

void ImageMxCon::newLab2(bool l) {
emit(lab2Toggle((int)l));
}

void ImageMxCon::newHist(bool t) {
histvis=t;
if (t) {
	if (data) imageChanged(data);
	if (datal) imagesChanged(datal);
	setFixedSize(390,215);
}
else {
	setFixedSize(390,115);
}

}

void ImageMxCon::newPoint(int x,int y) {

}

void ImageMxCon::setModeApp() {
emit(newMode(0));
}

void ImageMxCon::setModeProbe() {
emit(newMode(4));
}

void ImageMxCon::setModeDel() {
emit(newMode(1));
}

void ImageMxCon::setModeMove() {
emit(newMode(2));
}

void ImageMxCon::setModeSplit() {
emit(newMode(3));
}

void ImageMxCon::imageChanged(EMData *in) {
data=in;
datal=NULL;
if (histvis) {
	hist->setData(in);
}
}

void ImageMxCon::imagesChanged(QList<EMData> *im) {
showfft->hide();
datal=im;
data=NULL;
if (histvis) {
	hist->setData(im,subim);
}
}

void ImageMxCon::imageMinMax(float min,float max) {
hist->setMinMax(min,max);
}
