// align3d.C - 06/15/98 Steve Ludtke
// Copyright 1998 Baylor College of Medicine
// This program will rotationally and translationally align 2 3d models.
// It uses a simple grid search rather than r/t CCF's, this makes it
// somewhat slow. Eventually it should probably be rewritten

//N align3d
//F Rotational and translational alignment of 3d models
//T LA
//1
//P <fixed input>	The reference to align to
//P <input>	Volume to be aligned
//P <output>	Aligned output
//P [checkflip]	Allow for possible handedness difference between models
//P [trans]	Do a translational alignment at the end. Works better if models are precentered
//P [refine]	Assumes the models are already within a few degrees of proper alignment
//P [slow]	For models that seem to be difficult to align, this will search with a finer angular step
//P [noshrink]	Normally the models are scaled down by a factor of 2 for faster alignment. This causes the full sized model to be used.
//P [p3d]	Generate a 3d dataset containing the correlations vs the 3 euler angles
//P [saveref]	Experimental
//P [precen]	Experimental
//U align3d <fixed input> <free input> <output> [p3d] [refine] [checkflip] [saveref] [slow] [precen]
//D This program will take one 3d model and optimally align it to another 'reference' model.
//E Rotational alignment is performed by testing all possible angles at progressively smaller
//E angular increments. If the initial increment is too large, it may fail for some models.
//E The slow option will reduce the initial increment to try to take care of this problem.
//E Eventually this routine will be rewritten to use rotational cross-correlation. 

#include <stdio.h>
#include <math.h>
#include "EMData.h"
#include "List.h"
#include "util.h"

inline float asub(float x,float y) {
float r;

r=fabs(x-y);
if (r>PI) r=2.0*PI-r;

return r;
}

main(int argc, char *argv[]) {
int i,Ref;
EMData *d1,*d2,*d3;
float alt,az,phi,lc,best,eul[3],eul2[3],fbest,feul[3],step=20.0;
int dot=1,flip=0,refine=0,saveref=0,precen=0,trans=0,noshr=0;
FILE *p3d=NULL;

appinit(argc,argv);

if (argc<4) appusage(argv[0]);


for (i=4; i<argc; i++) {	
//	if (strncmp(argv[i],"dot",3)==0) dot=1;
	if (strncmp(argv[i],"p3d",3)==0) p3d=fopen("a3d.p3d","wb");
	else if (strncmp(argv[i],"checkflip",9)==0) flip=1;
	else if (strncmp(argv[i],"precen",6)==0) precen=1;
	else if (strncmp(argv[i],"trans",5)==0) trans=1;
	else if (strncmp(argv[i],"refine",6)==0) refine=1;
	else if (strncmp(argv[i],"slow",4)==0) step=9.0;
	else if (strncmp(argv[i],"saveref",7)==0) saveref=1;
	else if (strncmp(argv[i],"noshrink",8)==0) noshr=1;
	else { printf("Unknown parameter '%s'\n",argv[i]); exit(1); }
}

printf("Read reference image\n");
d1=new EMData;
d1->readImage(argv[1],-1);
if (!noshr) d1->medianShrink(2);
d1->normalize();
if (!precen) d1->cmCenter(1);

printf("Read image to be aligned\n");
d2=new EMData;
d2->readImage(argv[2],-1);
if (!noshr) d2->medianShrink(2);
d2->normalize();
if (!precen) d2->cmCenter(1);
d2=d2->copy();

d3=d2->copy();
d3->vFlip();
d3=d3->copy();

Ref=LOGbegin(argc,argv);
LOG(Ref,argv[1],LOG_REFFILE,NULL);
LOG(Ref,argv[2],LOG_INFILE,NULL);

eul2[0]=eul2[1]=eul2[2]=0;
d2->setRAlign(0,0,0);
d2->rotateAndTranslate();
//best=d1->dot(d2);
best=0;

// should really put all this stuff into 1 loop...

if (!refine) {
printf("Align to +-%0.1f degrees\n",step);
// locate best alignment to +-step degrees
for (alt=step/2; alt<=180.0+step/2; alt+=step) {
	for (az=0; az<360.0; az+=step) {
		for (phi=step/2; phi<360.0+step/2; phi+=step) {
			d2->setRAlign(alt*PI/180.0,az*PI/180.0,phi*PI/180.0);
			d2->rotateAndTranslate();
//			lc=d1->dot(d2);
			lc=5.0-d1->lcmp(d2,1);
			if (p3d) fprintf(p3d,"%f\t%f\t%f\t%f\n",alt,az,phi,lc);
			if (lc>best) { best=lc; eul2[0]=phi; eul2[1]=alt; eul2[2]=az; }
		}
	printf("%f %f        \r",alt,az); fflush(stdout);
	}
}
if (p3d) fclose(p3d);
printf("Best %f %f %f\t%f\n",eul2[1],eul2[2],eul2[0],best);

if (flip) {
	feul[0]=feul[1]=feul[2]=0;
	d3->setRAlign(0,0,0);
	d3->rotateAndTranslate();
//	best=d1->dot(d3);
	best=0;

// should really put all this stuff into 1 loop...

	printf("Align to +-%0.2f degrees\n",step);
// locate best alignment to +-30 degrees
	for (alt=step/2; alt<=180.0+step/2; alt+=step) {
		for (az=0; az<360.0; az+=step) {
			for (phi=step/2; phi<360.0+step/2; phi+=step) {
				d3->setRAlign(alt*PI/180.0,az*PI/180.0,phi*PI/180.0);
				d3->rotateAndTranslate();
//				lc=d1->dot(d3);
				lc=5.0-d1->lcmp(d2,1);
				if (p3d) fprintf(p3d,"%f\t%f\t%f\t%f\n",alt,az,phi,lc);
				if (lc>best) { fbest=lc; feul[0]=phi; feul[1]=alt; feul[2]=az; }
			}
		printf("%f %f        \r",alt,az); fflush(stdout);
		}
	}
	printf("Best flipped %f %f %f\t%f\n",feul[1],feul[2],feul[0],fbest);

	// flipped model better than unflipped?
	if (fbest<best) {
		printf("Model flipped\n");
		delete d2;
		d2=d3;
		d3=NULL;
//		d2->vFlip();
		eul2[0]=feul[0];
		eul2[1]=feul[1];
		eul2[2]=feul[2];
		best=fbest;
	}
}
}
else { eul2[0]=eul2[1]=eul2[2]=0; best=0; }

while (step>.5) {
step/=3.8;

eul[0]=eul2[0];
eul[1]=eul2[1];
eul[2]=eul2[2];
printf("Align to +-%0.2f degrees\n",step);
// locate best alignment to +-6 degrees
for (alt=eul[1]-step*4; alt<eul[1]+step*4.5; alt+=step) {
	for (az=eul[2]-step*4; az<eul[2]+step*4.5; az+=step) {
		for (phi=eul[0]-step*4; phi<eul[0]+step*4.5; phi+=step) {
			d2->setRAlign(alt*PI/180.0,az*PI/180.0,phi*PI/180.0);
			d2->rotateAndTranslate();
//			d2->normalize();
//			lc=d1->dot(d2);
			lc=5.0-d1->lcmp(d2,1);
			if (lc>best) { best=lc; eul2[0]=phi; eul2[1]=alt; eul2[2]=az; }
		}
	printf("%f %f        \r",alt,az); fflush(stdout);
	}
}
printf("Best %f %f %f\t%f\n",eul2[1],eul2[2],eul2[0],best);

}

if (saveref) {
	d1->writeMRC("d1.mrc");
	d2->setRAlign(eul2[1]*PI/180.0,eul2[2]*PI/180.0,eul2[0]*PI/180.0);
	d2->rotateAndTranslate();
	d2->writeMRC("d2.mrc");
}

d2->readImage(argv[2],-1);
d2=d2->copy();
if (trans) {
	d2->setRAlign(eul2[1]*PI/180.0,eul2[2]*PI/180.0,eul2[0]*PI/180.0);
	d2->setTAlign(0,0,0);
	d2->rotateAndTranslate();
	d2->normalize();
	d2->transAlign3d(d1,0,0);
	d2->setRAlign(eul2[1]*PI/180.0,eul2[2]*PI/180.0,eul2[0]*PI/180.0);
	printf("****** r: %f %f %f\tt:%f %f %f\n",d2->phi()*180.0/PI,d2->alt()*180.0/PI,d2->az()*180.0/PI,d2->Dx(),d2->Dy(),d2->Dz());
}
else {
	d2->setTAlign(0,0,0);
	d2->setRAlign(eul2[1]*PI/180.0,eul2[2]*PI/180.0,eul2[0]*PI/180.0);
	printf("****** r: %f %f %f\n",d2->phi()*180.0/PI,d2->alt()*180.0/PI,d2->az()*180.0/PI);
}
d2->rotateAndTranslate();
d2->normalize();

eul[0]=eul2[0];
eul[1]=eul2[1];
eul[2]=eul2[2];

d2->writeMRC(argv[3]);
LOG(Ref,argv[3],LOG_OUTFILE,NULL);
LOG(Ref,NULL,LOG_END,NULL);
}
