// parallelized using pthreads by Wen Jiang, April 6, 1999
// 		
// helixhunter.c Copyright 1999 Steve Ludtke
// 1/29/99
// Copyright 1999 Baylor College of Medicine
// This program will search a med-res (~8A) map for alpha helices

//N helixhunterP
//F This program is used to locate possible helices in intermediate resolution maps. Parallelized.
//T X
//1
//P <input>	Input 3D map at 5-8A resolution
//P <output>	Output 3d map of helix positions
//P <a/pix>	Angstroms/pixel in 3D map. Must be <2.
//P [cyl]	Also outputs a 3d map containing the cylinder used in the search.
//P [int=<file>]	Outputs the raw correlation files describing helix locations.
//P [da=<angle>]	Angular step used in the hunt.
//P [keep=<sigmamult>]	Specifies the peak strength that defines a helix.
//P [filt=<res>]	Prefilter resolution in angstroms.
//P [proc=<n>]	Number of processors to use.
//U	helixhunterP input.mrc output.mrc 1.5 proc=4
//D This is a parallelized version of <a href=helixhunter.html>helixhunter</a>. It is currently
//E parallelized using pthreads, and will run only on shared memory machines.
//D This is superseded by <a href="helixhunter2.html">helixhunter2</a>
//D	Visit web page <a href="http://ncmi.bcm.tmc.edu/~wjiang/hhfh">Helixhunter and Foldhunter</a> for 
//E	more information.

#include <stdio.h>
#include <math.h>
#include "EMData.h"
#include <pthread.h>
#include <assert.h>
#include <float.h>				

int NUMPROC = 1,nz,kcyl=0,ps,ps2,TIME=0;
EMData **best, **balt, **baz;
EMData **model;
float DA=-1, apix;
#define HLXL    (15.0/2.0)
//#define HLXW  (7.0/2.0)

void* helix_hunter_thread(void *arg){
int ithread=*(int*)arg;
EMData *cyl;		// local, each thread has its own cyl

cyl=new EMData;
cyl->setSize(nz,nz,nz);

// best stores the  highest ccf coeffiecient
best[ithread]->setSize(nz,nz,nz);	// global, each thread has its own one in the array
best[ithread]->zero();

// balt stores the corresponding alt angle for the cyl
balt[ithread]->setSize(nz,nz,nz);
balt[ithread]->zero();

// baz stores the corresponding az angle for the cyl
baz[ithread]->setSize(nz,nz,nz);
baz[ithread]->zero();

if(ithread!=0)model[ithread]=model[0]->copy();

//model->toCorner();
float from, to;		// alt angl bounds for each processor 
//from=PI/NUMPROC*ithread;
//to=PI/NUMPROC*(ithread+1)+(DA/2.0);

float *d,*d2,*dalt,*daz, alt, az,dlaz,x,y,z,dl;
int i, j, k, l;
//for (alt=from; alt<to; alt+=DA) {
for (alt=ithread*DA; alt<PI+DA/2.0; alt+=DA*NUMPROC) { //cyclic distribution
	i=floor(sin(alt)*PI/DA);
	dlaz=PI/(float)(i+1);
	for (az=0; az<PI-dlaz/2; az+=dlaz) {
		// generate cylinder for ccf
		cyl->zero();
		d=cyl->getData();
		
		for (k=-ps; k<=ps; k++) {		// this loop synthesize the cyl: centered, but oriented at (alt, az)
			for (j=-ps; j<=ps; j++) {
				for (i=-ps; i<=ps; i++) {
					l=(i+nz/2)+(j+nz/2)*nz+(k+nz/2)*nz*nz;
					x= i*cos(az)         +j*sin(az);
					y=-i*sin(az)*cos(alt)+j*cos(alt)*cos(az)+k*sin(alt);
					z= i*sin(alt)*sin(az)-j*sin(alt)*cos(az)+k*cos(alt);
					dl=hypot(x*apix,y*apix);
					if (z>ps2) d[l]=exp(-(SQR(dl/6.0)+SQR((z-ps2)*apix/2.0)))*cos(dl*PI/5.0);
					else if (z<-ps2) d[l]=exp(-(SQR(dl/6.0)+SQR((z+ps2)*apix/2.0)))*cos(dl*PI/5.0);
					else d[l]=exp(-SQR(dl/6.0))*cos(dl*PI/5.0);
				}
			}
		}

		cyl->doneData();
		if (kcyl && az==0 && alt==0 && ithread==0)cyl->writeMRC("cyl.mrc");
//              cyl->toCorner();
		
		// now do a CCF
		EMData* ccf=model[ithread]->calcCCF(cyl,1);
		ccf->multConst(1.0e9);
//		ccf->toCorner();
//              if (ccf->Max()<fabs(ccf->Min())) { ccf->multConst(-1.0); printf("Inv  "); }
//              else printf("     ");

//              char t[80];
//              sprintf(t,"ccf.%1.0f.%1.0f.mrc",alt*180/PI,az*180/PI);
//              cyl->writeMRC(t);

		// Now keep track of the best cylinder at each voxel
		d=ccf->getDataRO();
		d2=best[ithread]->getData();
		dalt=balt[ithread]->getData();
		daz=baz[ithread]->getData();

		for (k=0; k<nz; k++) {
			for (j=0; j<nz; j++) {
				for (i=0; i<nz; i++) {                                  
					if (*d>*d2) {
						*d2=*d;
						*dalt=alt;
						*daz=az;
					}
					d++;
					d2++;
					dalt++;
					daz++;
				}
			}
		}
		ccf->doneData();
		best[ithread]->doneData();
		balt[ithread]->doneData();
		baz[ithread]->doneData();

//              char t[80];
//              sprintf(t,"ccf.%d.mrc",nn++);
//              best->writeMRC(t);

		if(!TIME)printf("thread=%d\t%5.1f,%5.1f\tccf max=%g sig=%g\n",ithread,alt*180.0/PI,az*180.0/PI,ccf->Max(),ccf->Sigma());
//              printf("%f\t%f\r",alt*180.0/PI,az*180.0/PI); fflush(stdout);
		delete ccf;
	}
}
return NULL;
}


int main(int argc, char *argv[]) {
EMData *out;
int i,j,k;
int ii,jj,kk,nn=0;
float x,y,z,s,bm,bs,sigallow=0.5;
float *dt,*dt2,*dt3;
float *a[4],dl;
char *intfile=NULL;
time_t total_time=0, tmp_time;
int ithread;
float res=-1;
		
if (argc<4) appusage(argv[0]);

for (i=4; i<argc; i++) {    
	if (strncmp(argv[i],"int=",4)==0) intfile=&argv[i][4];
	else if (strncmp(argv[i],"cpu=",4)==0) {
		NUMPROC=atoi(&argv[i][4]);
		if(NUMPROC<=0)NUMPROC=1;
	}
	else if (strncmp(argv[i],"da=",3)==0) assert(DA=atof(&argv[i][3])*PI/180.0);
	else if (strncmp(argv[i],"cyl",3)==0) kcyl=1;
	else if (strncmp(argv[i],"keep=",5)==0)assert(sigallow=atof(&argv[i][5]));
	else if (strncmp(argv[i],"res=",4)==0) res=atof(&argv[i][4]);
	else if (strncmp(argv[i],"proc=",5)==0) NUMPROC=atoi(argv[i]+5);
	else if (strncmp(argv[i],"tim",3)==0) TIME=1;
	else { printf("Unknown parameter '%s'\n",argv[i]); exit(1); }
}

assert(apix=atof(argv[3]));
if (apix<=0) { fprintf(stderr,"A/pix must be >=0\n"); exit(1); }
if (apix>2) { fprintf(stderr,"Helixhunter will not work properly if A/pix > 2\n"); /*exit(1);*/ }

tmp_time=time(NULL);	// monitor the starting time

NUMPROC=(NUMPROC>8)?8:NUMPROC;  // limit to at most 8 processors
assert(model=(EMData**)malloc(sizeof(EMData*)*NUMPROC));
assert(best=(EMData**)malloc(sizeof(EMData*)*NUMPROC));
assert(balt=(EMData**)malloc(sizeof(EMData*)*NUMPROC));
assert(baz=(EMData**)malloc(sizeof(EMData*)*NUMPROC));

for(i=0;i<NUMPROC;i++){
	best[i]=new EMData;
	balt[i]=new EMData;
	baz[i]= new EMData;
	model[i]=new EMData;
}

total_time+=time(NULL)-tmp_time;

if (model[0]->readImage(argv[1],-1)) { printf("Error reading image\n"); exit(1); }
model[0]->normalize();
nz=model[0]->zSize();
if (nz!=model[0]->xSize() || nz!=model[0]->ySize()) {
	printf("The model MUST be in a square box (nx=ny=nz)\n");
	exit(1);
}

if(DA<=0){
	if(res<=0) res=8;
	DA=2*asin(res/nz);
	printf("da is set to %.1f degree\n",DA*180/PI);
}

ps=(int)floor(HLXL/apix*2);
ps2=(int)floor(HLXL/apix);

int Ref=LOGbegin(argc,argv);
LOG(Ref,argv[1],LOG_INFILE,NULL);


tmp_time=time(NULL);

// start the actual ccf search in parallel
pthread_t *thrd;
thrd=(pthread_t *)malloc((sizeof(pthread_t*))*NUMPROC);
for (ithread=0; ithread < NUMPROC;  ithread++){
	if((pthread_create(&thrd[ithread],NULL,helix_hunter_thread,(void *)ithread))!=0){
		printf("Cannot creat thread %d\n",ithread);
		exit(-1);
	}
}

for (ithread=0; ithread < NUMPROC; ithread++) pthread_join(thrd[ithread],NULL); // wait until all threads finish

	
// find the global maxiamal CCF coefficient by merging the NUMPROC best, balt, baz cubes into one best[0], balt[0], baz[0]
//
// the later steps are pretty fast, I don't bother to paralleize them 
{
float **d2,**dalt,**daz, tmp;
d2=(float**)malloc(sizeof(float*)*NUMPROC);
dalt=(float**)malloc(sizeof(float*)*NUMPROC);
daz=(float**)malloc(sizeof(float*)*NUMPROC);

for(i=0;i<NUMPROC;i++){
	d2[i]=best[i]->getData();
	dalt[i]=balt[i]->getData();
	daz[i]=baz[i]->getData();
}

for (k=0; k<nz; k++) { // loop throught all nz^3 voxels
	for (j=0; j<nz; j++) {
		for (i=0; i<nz; i++) {                                  
			tmp=FLT_MIN;               
			for(ithread=0;ithread<NUMPROC;ithread++){	// for each voxel, find the best orientation
				if (*d2[ithread]>tmp) {
					tmp=*d2[ithread];
					*d2[0]=*d2[ithread];
					*dalt[0]=*dalt[ithread];
					*daz[0]=*daz[ithread];
				}
				d2[ithread]++;
				dalt[ithread]++;
				daz[ithread]++;
			}
		}
	}
}

for(i=0;i<NUMPROC;i++){
	best[i]->doneData();
	balt[i]->doneData();
	baz[i]->doneData();
}

if(NUMPROC>1)
	for(i=1;i<NUMPROC;i++){
		delete model[i], best[i], balt[i], baz[i];
	}
}

total_time+=time(NULL)-tmp_time;

if(!TIME)printf("\nccf max=%g sig=%g\n",best[0]->Max(),best[0]->Sigma());

best[0]->multConst(1.0/best[0]->Sigma());
model[0]->filter(-5,apix*model[0]->ySize()/8.0,2);
best[0]->add(model[0]);
model[0]->realFilter(2,best[0]->Mean()+best[0]->Sigma()*sigallow); //binary masking using the second variable
best[0]->mult(model[0]);


bm=best[0]->Max();
bs=best[0]->Sigma();

if (intfile) {
	best[0]->writeMRC(intfile);
	balt[0]->writeMRC(strcat(intfile,".alt"));
	baz[0]->writeMRC(strcat(intfile,".az"));
	LOG(Ref,intfile,LOG_OUTFILE,"intermediate");
}

out=new EMData;
out->setSize(nz,nz,nz);
out->zero();

// Now make a map out of the best cylinders
a[0]=out->getData();
a[1]=best[0]->getDataRO();
a[2]=balt[0]->getDataRO();
a[3]=baz[0]->getDataRO();
for (s=bm-bs*8.0; s<=bm+bs/4.0; s+=bs/2.0) {
//for (s=bm-bs/2.0; s<=bm+bs/4.0; s+=bs/2.0) {
float *d=a[0];
float *d2=a[1];
float *dalt=a[2];
float *daz=a[3];
for (k=0; k<nz; k++) {
	for (j=0; j<nz; j++) {
		for (i=0; i<nz; i++) {
			// if we have a big peak, draw a line in the output file
			if (*d2>s) {
				dt3=NULL;
				for (dl=-ps/2; dl<=ps/2; dl+=.25) {
					ii=dl*sin(*dalt)*sin(*daz);
					jj=-dl*sin(*dalt)*cos(*daz);
					kk=dl*cos(*dalt);
					if (i+ii<0||i+ii>=nz || j+jj<0||j+jj>=nz || k+kk<0||k+kk>=nz) continue;
					dt2=d+(ii)+(jj)*nz+(kk)*nz*nz;
					if (dt2==dt3) continue;
					dt3=dt2;
					(*dt2)=s;
				}
			}
			d++;
			d2++;
			dalt++;
			daz++;
		}
	}
}
}
out->doneData();
best[0]->doneData();
balt[0]->doneData();
baz[0]->doneData();

out->writeMRC(argv[2]);	//output the final map with the best helices
LOG(Ref,argv[2],LOG_OUTFILE,NULL);
LOG(Ref,NULL,LOG_END,NULL);

if(TIME)printf("NumProcessors=%3d\tSearchingTime(second)=%5d\tVolumeSize=%3d^3\n",NUMPROC,total_time,out->xSize());

}
